# Copyright (c) 2013 Vinay Bettadapura (Georgia Tech)
#
# This script generates the confusion matrix given the match file, label file
# and the number of nearest neighbors to compare against

if ARGV.size != 3
	puts "\nUsage: ruby generate_confusion_matrix.rb <match_file> <label_file> <k-nearest-neighbors>\n\n"
	exit
end

def print_table(a, cs=9)
	return if a.empty?
	report = []   
	report << a.enum_for(:each_with_index).map{|ia,i|
		"%s " % [ia.map{|e| "%#{cs}s" % e}.join(" ") ]
	}
	puts report.join("\n")
end

def compute()
	match_file = ARGV[0]
	label_file = ARGV[1]
	num_matches = ARGV[2].to_i

	fp = File.open(label_file, "r")
	line_1 = fp.gets.chomp.split(" ")
	line_2 = fp.gets.chomp.split(" ")
	fp.close

	label_defs = {}
	label_ids = {}
	line_2.each{|l|
		l = l.split(":")
		label_defs[l[0]] = l[1]
		label_ids[l[1]] = l[0]
	}

	labels = {}
	line_1.each{|l|
		l = l.split(":")
		labels[l[0]] = label_defs[l[1]]
	}

	ids = []
	results = []

	File.open(match_file, "r").each_line do |line|
		line = line.chomp.split(" ")
	
		track_id = line.shift
		ids << track_id
	
		map = {}
		for i in 0..line.size-1
			affinity_track_id = line[i].split(":")[0]
			affinity_val = line[i].split(":")[1].to_f
			map[affinity_val] = affinity_track_id
		end

		sorted_keys = map.keys.sort_by{|n| -n}

		tmp_result = []
		count = 1
		sorted_keys.each{|k|
		next if map[k] == track_id
		if count > num_matches
			break
		else
			tmp_result << labels[map[k]]
			count += 1
		end
		}
		results << tmp_result.group_by{|e| tmp_result.count(e)}.max[1].uniq[0]
	end

	confusion_matrix = {}
	total_num = {}
	label_defs.each{|k,v|
		tmp = {}
		label_defs.each{|kk,vv|
			tmp[vv] = 0
		}
		confusion_matrix[v] = tmp
		total_num[v] = 0
	}

	for index in 0..results.size-1
		actual_label = labels[ids[index]]
		result_label = results[index]
		confusion_matrix[actual_label][result_label] += 1
	end

	labels.each{|k,v|
		total_num[v] += 1
	}

	# Delete those labels that have no data-points associated with them
	confusion_matrix.delete_if{|k,v| total_num[k] == 0}
	label_ids.delete_if{|k,v| total_num[k] == 0}
	total_num.delete_if{|k,v| v == 0}

	table = []
	num_table = []
	keys = confusion_matrix.keys.sort
	keys.each{|k|
		sub_table = [k]
		num_sub_table = []
		keys.each{|kk|
			val = confusion_matrix[k][kk].to_i
			num_sub_table << val

			final_val = (val/total_num[k].to_f * 10000).round/100.0
			sub_table << final_val
		}
		num_table << num_sub_table
		table << sub_table
	}

	correct = 0
	for i in 0..num_table.size-1
		for j in 0..num_table[i].size-1
			if i == j
				correct += num_table[i][j]
			end
		end
	end

	avg_accuracy = (correct/ids.size.to_f * 10000).round/100.0

	puts "\n------------------------------------------------------\n"
	puts "CONFUSION MATRIX:\n"
	print_table(table, 9)
	puts "\nAVERAGE ACCURACY: #{avg_accuracy}\n"
	puts "------------------------------------------------------\n\n"

	return avg_accuracy
end

compute()
