// Copyright (c) 2013 Vinay Bettadapura (Georgia Tech)

#include <boost/regex.hpp>
#include <iostream>
#include <string>
#include <vector>

#include "reg_exp_gen_tbb.h"
#include "reg_exp_generator.h"

using std::cout;
using std::endl;
using std::string;
using std::vector;

namespace core {

void RegExpGenTBB::operator() (const tbb::blocked_range<int>& r) const
{
	for(int i = r.begin(); i < r.end(); ++i)
	{
		RegExpGenerator reg_exp_gen(v_all_symbols_, v_all_symbols_unshuffled_,
			v_symbol_cdf_, s_generated_regexps_, num_symbols_);

		string str_reg_exp = "";

		bool match = false;
		while(!match)
		{
			str_reg_exp = reg_exp_gen.Get();
			boost::regex reg_exp(str_reg_exp);

			vector<string>::size_type num_matches = 0;
			for(vector<string>::size_type j = 0;
				j < v_event_strings_->size(); ++j)
			{
				try
				{
					if(boost::regex_match((*v_event_strings_)[j], reg_exp))
					{
						match = true;
						++num_matches;
					}
				}
				catch(const std::exception& e)
				{
					cout << "Caught error: " << e.what() << "\n";

					num_matches = 0;
					match = false;
					break;
				}
			}
			
			if(num_matches == v_event_strings_->size())
			{
				num_matches = 0;
				match = false;
			}
		}
		
		(*v_random_reg_exps_)[i] = str_reg_exp;

		cout << "Reg exp #" << i + 1 << ": " << str_reg_exp << endl;
	}
}

}
