// Copyright (c) 2013 Vinay Bettadapura (Georgia Tech)

#include <algorithm>
#include <cstdlib>

#include "reg_exp_generator.h"

using std::set;
using std::string;
using std::vector;

namespace core {

RegExpGenerator::RegExpGenerator(
	vector<string> symbols,
	const vector<string>* symbols_unshuffled,
	const vector<double>* symbol_cdf,
	set<string>* generated_regexps,
	unsigned num
) : 
	v_symbols_(symbols),
	v_symbols_unshuffled_(symbols_unshuffled),
	v_symbol_cdf_(symbol_cdf),
	s_generated_regexps_(generated_regexps),
	N_(num)
{
	v_special_chars_.push_back("*");
	v_special_chars_.push_back("+");
	v_special_chars_.push_back("?");

	srand(time(0));
}

template<class bidiiter>
bidiiter RegExpGenerator::random_unique(bidiiter begin,
										bidiiter end,
										size_t num_random)
{
	size_t left = std::distance(begin, end);

	while(num_random--)
	{
		bidiiter r = begin;
		std::advance(r, rand() % left);
		std::swap(*begin, *r);

		++begin;
		--left;
	}
	
	return begin;
}

int RegExpGenerator::GetRand(int min, int max)
{
	int random = ((double) rand() / RAND_MAX) * (max - min + 1) + min;
	random = random > max ? max : random;

	return random;
}

double RegExpGenerator::GetRandBetweenZeroAndOne()
{
	return (double)rand() / (double)(RAND_MAX - 1);
}

void RegExpGenerator::GetSymbols(string& reg_ex, int num)
{
	int num_symbols = GetRand(1, num);

	random_unique(v_symbols_.begin(), v_symbols_.end(), num_symbols);

	for(int i = 0; i < num_symbols; ++i)
	{
		reg_ex += v_symbols_[i];

		if(i != num_symbols - 1)
			reg_ex += "|";
	}
}

void RegExpGenerator::GetProbSymbol(string& reg_ex)
{
	double prob = GetRandBetweenZeroAndOne();
	
	vector<double>::const_iterator up = std::upper_bound(v_symbol_cdf_->begin(),
													v_symbol_cdf_->end(), prob);
	
	int position = int(up - v_symbol_cdf_->begin());

	reg_ex += (*v_symbols_unshuffled_)[position];
}

string RegExpGenerator::GetHelper()
{
	string reg_ex = "^.*(";
	
	GetProbSymbol(reg_ex);

	reg_ex += ")(";

	GetSymbols(reg_ex, N_);

	reg_ex += ")" + v_special_chars_[GetRand(0, v_special_chars_.size()-1)] + "(";

	GetProbSymbol(reg_ex);

	reg_ex += ").*$";

	return reg_ex;
}

string RegExpGenerator::Get()
{
	string reg_ex = "";
	while(s_generated_regexps_->find(reg_ex) != s_generated_regexps_->end())
	{
		reg_ex = GetHelper();
	}
	s_generated_regexps_->insert(reg_ex);

	return reg_ex;
}

}
