// Copyright (c) 2013 Vinay Bettadapura (Georgia Tech)

#include <cmath>

#include "vector_space_similarity_tbb.h"

using std::map;
using std::string;
using std::vector;

namespace core {

float VectorSpaceSimilarityTBB::ComputeVectorMagnitude(
											unsigned long long i) const
{
	float mag = 0.0;

	unsigned long long begin = no_of_bins_ * i;

	for(unsigned long long i = 0; i < no_of_bins_; ++i)
	{
		mag += pow(a_ngram_hist_[begin + i], 2);
	}
	mag = sqrt(mag);
	
	return mag;
}

float VectorSpaceSimilarityTBB::ComputeDotProduct(unsigned long long i,
												unsigned long long j) const
{
	unsigned long long begin_i = no_of_bins_ * i;
	unsigned long long begin_j = no_of_bins_ * j;
	
	float result = 0.0;
	for(unsigned long long iter = 0; iter < no_of_bins_; ++iter)
	{
		result += a_ngram_hist_[begin_i + iter] * a_ngram_hist_[begin_j + iter];
	}

	return result;
}
	
float VectorSpaceSimilarityTBB::ComputeSimilariyUsingVSMMethod(
							unsigned long long i, unsigned long long j) const
{
	float mag_a = 0.0, mag_b = 0.0, result = 0.0;

	mag_a = ComputeVectorMagnitude(i);
	mag_b = ComputeVectorMagnitude(j);

	if(mag_a == 0 || mag_b == 0)
	{
		return 0;
	}

	result = ComputeDotProduct(i, j);

	return result/(mag_a * mag_b);
}

void VectorSpaceSimilarityTBB::operator() (
						const tbb::blocked_range<unsigned long long>& r) const
{
	float affinity = 0.0;

	for(unsigned long long i = r.begin(); i != r.end(); ++i)
	{
		for(unsigned long long j = i; j < no_of_tracks_; ++j)
		{
			if(j == i)
			{
				affinity = 1.0;
			}
			else if(j > i)
			{
				affinity = ComputeSimilariyUsingVSMMethod(i, j);
			}
			else
			{
				return;
			}

			a_affinity_[i * no_of_tracks_ + j] = affinity;
			a_affinity_[j * no_of_tracks_ + i] = affinity;
		}
	}
}

}
