// Copyright (c) 2013 Vinay Bettadapura (Georgia Tech)

#ifndef _CORE_VECTOR_SPACE_SIMILARITY_TBB_
#define _CORE_VECTOR_SPACE_SIMILARITY_TBB_

#include <boost/shared_array.hpp>
#include <map>
#include <string>
#include <tbb/blocked_range.h>
#include <tbb/parallel_for.h>
#include <vector>

namespace core {

class VectorSpaceSimilarityTBB
{
public:
	VectorSpaceSimilarityTBB(
		boost::shared_array<float> ngram_hist,
		boost::shared_array<float> affinity,
		unsigned long long no_of_tracks,
		unsigned long long no_of_bins
	) 
	: a_ngram_hist_(ngram_hist),
	a_affinity_(affinity),
	no_of_tracks_(no_of_tracks),
	no_of_bins_(no_of_bins) { }

	void operator() (const tbb::blocked_range<unsigned long long>& r) const;

private:
	boost::shared_array<float> a_ngram_hist_;
	boost::shared_array<float> a_affinity_;
	unsigned long long no_of_tracks_;
	unsigned long long no_of_bins_;

	float ComputeVectorMagnitude(unsigned long long i) const;
	float ComputeDotProduct(unsigned long long i, unsigned long long j) const;
	float ComputeSimilariyUsingVSMMethod(unsigned long long i,
										unsigned long long j) const;
};

}

#endif
