# Copyright (c) 2013 Vinay Bettadapura (Georgia Tech)
# 
# Use this script to validate your events.txt file. The script checks your
# events.txt file for common errors.

if ARGV.size != 1
	puts "\nUsage: ruby event_file_validator.rb <event_file>\n\n"
	exit
end

event_file = ARGV[0]

if event_file.split("/").last != "events.txt"
	puts "\nError: Event filename is not \"events.txt\". Please see the README file.\n\n"
	exit
end

num_lines = 0
previous_line = []
File.open(event_file, "r").each_line do |line|
	line = line.chomp.split(" ").collect{|i| i.to_f}
	num_lines = num_lines + 1
	
	if line.size != 6
		puts "\nError: Line #{num_lines} of the event file does not have the expected number of fields (expected 6). Please see the README file.\n\n"
		exit
	end
	
	if line[3] < line[2]
		puts "\nError: End frame number of the event in line #{num_lines} is lesser than its start frame number. Please see the README file.\n\n"
		exit
	end
	
	if line[5] < line[4]
		puts "\nError: End time of the event in line #{num_lines} is lesser than its start time. Please see the README file.\n\n"
		exit
	end

	if num_lines == 1
		previous_line = line
		next
	end

	if line[0] == previous_line[0] and line[2] < previous_line[2]
		puts "\nError: Start frame number of the event in line #{num_lines} is lesser than the start frame number of the event in line #{num_lines-1}. Please see the README file.\n\n"
		exit
	end

	if line[0] == previous_line[0] and line[4] < previous_line[4]
		puts "\nError: Start time of the event in line #{num_lines} is lesser than the start time of the event in line #{num_lines-1}. Please see the README file.\n\n"
		exit
	end

	previous_line = line
end

if num_lines == 0
	puts "\nError: Event file is empty. Please see the README file.\n\n"
	exit
end

puts "\nSuccess: Valid event file\n\n"
